#include "testApp.h"
#include "particle.h"

#include "ofMain.h"

// comparison routine for sort...
bool sortX(  particle * a, particle * b ) { 
	return a->pos.x < b->pos.x; 
}              

bool sortY(  particle * a, particle * b ) { 
	return a->pos.y < b->pos.y; 
}              

bool sortRandom(  particle * a, particle * b ) { 
    // sort by memory address
	return (int)a < (int)b;
}              


//--------------------------------------------------------------
void testApp::setup(){

    for (int i = 0; i < 400; i++){
        particle * p = new particle();
        myParticles.push_back(p);
        myParticles[i]->pos.set(ofRandom(0,ofGetWidth()),ofRandom(0,ofGetHeight()));
        myParticles[i]->vel.set(0,0);
    }
    
    ofSetWindowTitle("Get me out of here!");
	
	ofSetCircleResolution(6);
	ofSetRectMode(OF_RECTMODE_CENTER);
    
    ofEnableAlphaBlending();
    ofSetLineWidth(5);
    
    centroid.set(0, 0);
}

//--------------------------------------------------------------
void testApp::update(){
    float avgX = 0;
    float avgY = 0;
	
	//cout << ofGetFrameRate() << endl;
	
	// sort all the particle
	sort( myParticles.begin(), myParticles.end(), sortX );               // sort them!
	
	
	
	ofBackground(0,0,0); //Grey background, NY style
	
    for (int i = 0; i < myParticles.size(); i++){
        myParticles[i]->resetForce();
        float angle = ofNoise(ofGetFrameNum() * 0.01) * FOUR_PI;
        myParticles[i]->addForce(cos(angle) * 0.05, sin(angle) * 0.05);
    }
	
	
	for (int i = 0; i < myParticles.size(); i++){
		for (int j = i-1; j >= 0; j--){
			if ( fabs(myParticles[j]->pos.x - myParticles[i]->pos.x) >  5) break;
            if(myParticles[i]->pos.y < centroid.y) {
                myParticles[i]->addRepulsionForce( *myParticles[j], 10, 1 );
            } else {
                myParticles[i]->addAttractionForce( *myParticles[j], 10, 1 );
            }
		}
    }
	
	
    for (int i = 0; i < myParticles.size(); i++){
        float returnForce = myParticles[i]->pos.x > 0 && myParticles[i]->pos.x < ofGetWidth() && myParticles[i]->pos.y > 0 && myParticles[i]->pos.y < ofGetHeight() ? 0.01 : 1;
		myParticles[i]->addAttractionForce( 250, 250, 1500, returnForce);
		myParticles[i]->addRepulsionForce( mouseX,mouseY,50, 5);
		myParticles[i]->addDampingForce();
        myParticles[i]->update();
        
        avgX += myParticles[i]->pos.x;
        avgY += myParticles[i]->pos.y;
	}
    
    centroid.set(avgX/myParticles.size(), avgY/myParticles.size());
}

//--------------------------------------------------------------
void testApp::draw(){
	
	ofSetColor(255,255,255,8);
	// then draw:
	
    sort( myParticles.begin(), myParticles.end(), sortRandom );
	for (int i = 2; i < myParticles.size(); i++){
        float p = (float)i/myParticles.size();
        ofSetColor(255,255*p,255*p, 8);
        ofTriangle(myParticles[i-2]->pos.x, myParticles[i-2]->pos.y, myParticles[i-1]->pos.x, myParticles[i-1]->pos.y, myParticles[i]->pos.x, myParticles[i]->pos.y);
        //ofCircle(myParticles[i-1]->pos.x, myParticles[i-1]->pos.y, 5);
        //ofLine(myParticles[i-1]->pos.x, myParticles[i-1]->pos.y, myParticles[i]->pos.x, myParticles[i]->pos.y);
	 }
    
//    sort( myParticles.begin(), myParticles.end(), sortY );
//    for (int i = 1; i < myParticles.size(); i++){
//        ofSetColor(255,255,255, 4 * (float)i/myParticles.size());
//        ofLine(myParticles[i-1]->pos.x, myParticles[i-1]->pos.y, myParticles[i]->pos.x, myParticles[i]->pos.y);
//    }
    
    

}

//--------------------------------------------------------------
void testApp::keyPressed  (int key){
   
}
//--------------------------------------------------------------
void testApp::keyReleased  (int key){

}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){

}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){

}

//--------------------------------------------------------------
void testApp::mouseReleased(){
	
}
